VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "clsFileFind"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'*********'*********'*********'*********'*********'*********'*********'*********
'
' clsFileFind
' www.classicvb.com
'
' Written by David Crowell
'
' Released to the Public Domain
'
' An easier way to access the FindFirstFile and related API calls.  Uses a
' natural recordset-like method that most VB programmers already have adjusted
' to.
'
' No warranty of any kind is given, or to be implied on the quality
' or functionality of this software.
'
'*********'*********'*********'*********'*********'*********'*********'*********
Option Explicit

Private Const INVALID_HANDLE_VALUE = -1
Private Const FILE_ATTRIBUTE_ARCHIVE = &H20
Private Const FILE_ATTRIBUTE_DIRECTORY = &H10
Private Const FILE_ATTRIBUTE_HIDDEN = &H2
Private Const FILE_ATTRIBUTE_READONLY = &H1
Private Const FILE_ATTRIBUTE_SYSTEM = &H4

Private Type FILETIME
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type

Private Type SYSTEMTIME
    wYear As Integer
    wMonth As Integer
    wDayOfWeek As Integer
    wDay As Integer
    wHour As Integer
    wMinute As Integer
    wSecond As Integer
    wMilliseconds As Integer
End Type

Private Type WIN32_FIND_DATA
    dwFileAttributes As Long
    ftCreationTime As FILETIME
    ftLastAccessTime As FILETIME
    ftLastWriteTime As FILETIME
    nFileSizeHigh As Long
    nFileSizeLow As Long
    dwReserved0 As Long
    dwReserved1 As Long
    cFileName(0 To 259) As Byte
    cAlternateFileName(0 To 13) As Byte
End Type

Private Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindNextFile Lib "kernel32" Alias "FindNextFileA" (ByVal hFindFile As Long, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindClose Lib "kernel32" (ByVal hFindFile As Long) As Long
Private Declare Function FileTimeToLocalFileTime Lib "kernel32" (lpFileTime As FILETIME, lpLocalFileTime As FILETIME) As Long
Private Declare Function FileTimeToSystemTime Lib "kernel32" (lpFileTime As FILETIME, lpSystemTime As SYSTEMTIME) As Long

Private mFileData As WIN32_FIND_DATA
Private mhFind As Long
Private mstrStartingPath As String

Private mblnFilterDotFolders As Boolean
Private mblnEOF As Boolean
Private mlngFilterAttributes As Long
Private mlngFilterAttributesMask As Long

'*********'*********'*********'*********'*********'*********'*********'*********
' Public Properties
'*********'*********'*********'*********'*********'*********'*********'*********

' FilterDotFolders
' if true will not return folders named '..' or '.'
' default = true
Public Property Get FilterDotFolders() As Boolean
    FilterDotFolders = mblnFilterDotFolders
End Property
Public Property Let FilterDotFolders(ByVal NewValue As Boolean)
    mblnFilterDotFolders = NewValue
End Property

' FilterAttributes
Public Property Get FilterAttributes() As VbFileAttribute
    FilterAttributes = mlngFilterAttributes
End Property
Public Property Let FilterAttributes(ByVal NewValue As VbFileAttribute)
    mlngFilterAttributes = NewValue
End Property

' FilterAttributesMask
Public Property Get FilterAttributesMask() As VbFileAttribute
    FilterAttributesMask = mlngFilterAttributesMask
End Property
Public Property Let FilterAttributesMask(ByVal NewValue As VbFileAttribute)
    mlngFilterAttributesMask = NewValue
End Property

' EOF
' returns true if we are past the end of the list
Public Property Get EOF() As Boolean
    EOF = mblnEOF
End Property

' FileName
' returns just the filename & extension
' eg: clsFileFind.cls
Public Property Get FileName() As String
    FileName = mTrimNull(StrConv(mFileData.cFileName, vbUnicode))
End Property

' ShortFileName
' returns the DOS filename
' eg: CLSFIL~1.CLS
Public Property Get ShortFileName() As String
    ShortFileName = mTrimNull(StrConv(mFileData.cAlternateFileName, vbUnicode))
End Property

' FullPath
' returns the full path, uses long path names even if the original
' search spec didn't
Public Property Get FullPath() As String
    FullPath = mstrStartingPath & FileName
End Property

' FileSize
' returns file size as a Variant with subtype Decimal
' this allows file sizes of greater than 2GB
Public Property Get FileSize() As Variant

    Dim h As Variant
    Dim l As Variant
    
    h = CDec(mFileData.nFileSizeHigh) * CDec(&H10000) * CDec(&H10000)
    l = CDec(mFileData.nFileSizeLow)
    FileSize = CDec(h + l)
    
End Property

' FileSizeLow
' returns the lower 32bits of the file size
' this is much faster than the FileSize property above
' and will work perfectly if files are less than 2GB in size
Public Property Get FileSizeLow() As Long
    FileSizeLow = mFileData.nFileSizeLow
End Property

' ReadOnly
' returns true if the file is read only
Public Property Get ReadOnly() As Boolean
    ReadOnly = (mFileData.dwFileAttributes And FILE_ATTRIBUTE_READONLY)
End Property

' Hidden
' returns true if the file is hidden
Public Property Get Hidden() As Boolean
    Hidden = (mFileData.dwFileAttributes And FILE_ATTRIBUTE_HIDDEN)
End Property

' System
' returns true if the file is a system file
Public Property Get System() As Boolean
    System = (mFileData.dwFileAttributes And FILE_ATTRIBUTE_SYSTEM)
End Property

' Folder
' returns true if the entry is a folder
Public Property Get Folder() As Boolean
    Folder = (mFileData.dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY)
End Property

' Archive
' returns true if the file has the archive bit set
Public Property Get Archive() As Boolean
    Archive = (mFileData.dwFileAttributes And FILE_ATTRIBUTE_ARCHIVE)
End Property

' LastAccess
' returns time/date of last access (if supported by file system)
Public Property Get LastAccess() As Date
    LastAccess = mFileTimeToDate(mFileData.ftLastAccessTime)
End Property

' LastModified
' returns time/date of last modification
Public Property Get LastModified() As Date
    LastModified = mFileTimeToDate(mFileData.ftLastWriteTime)
End Property

' Created
' returns time/date of file creation
Public Property Get Created() As Date
    Created = mFileTimeToDate(mFileData.ftCreationTime)
End Property

'*********'*********'*********'*********'*********'*********'*********'*********
' Public Methods
'*********'*********'*********'*********'*********'*********'*********'*********

' Find
' Start the search
Public Sub Find(ByRef Path As String, ByRef SearchSpec As String)
    
    Dim SearchPath As String
    
    ' if an existing search is open, then close the handle
    If mhFind > 0 Then
        Call FindClose(mhFind)
    End If
    
    ' build a proper search path
    mstrStartingPath = mQualifyPath(Path)
    SearchPath = mstrStartingPath & SearchSpec
    
    ' start the search
    mhFind = FindFirstFile(SearchPath, mFileData)
    
    ' check if we've returned anything
    If mhFind = INVALID_HANDLE_VALUE Then
        mblnEOF = True
    Else
        mblnEOF = False
    End If
    
    ' if current file does not match criteria, then continue looking
    If Not mMatchFilter Then
        MoveNext
    End If
    
End Sub

Public Sub MoveNext()

    Dim r As Long
    
    ' loop until we find another file that meets
    ' the criteria, or until we reach the end
    Do
        r = FindNextFile(mhFind, mFileData)
        mblnEOF = (r = 0)
    Loop Until mMatchFilter Or mblnEOF

End Sub

Public Sub CloseFind()

    ' if an existing search is open, then close the handle
    If mhFind > 0 Then
        Call FindClose(mhFind)
    End If
    
End Sub

'*********'*********'*********'*********'*********'*********'*********'*********
' Private Methods
'*********'*********'*********'*********'*********'*********'*********'*********

' trim trailing nulls
Private Function mTrimNull(ByRef StrIn As String) As String

   Dim i As Long
   Dim r As String
   
   i = InStr(StrIn, vbNullChar)
   Select Case i
      Case Is > 1
         r = Left$(StrIn, i - 1)
      Case 1
         r = vbNullString
      Case 0
         r = StrIn
   End Select
   
   mTrimNull = r
   
End Function

Private Function mFileTimeToDate(ByRef TimeStamp As FILETIME) As Date

    Dim ST As SYSTEMTIME
    Dim FT As FILETIME
    Dim r As Date
    
    ' convert from UTC to local time
    Call FileTimeToLocalFileTime(TimeStamp, FT)
    
    ' convert from filetime to systemtime
    Call FileTimeToSystemTime(FT, ST)
    
    ' convert to VB date
    r = DateSerial(ST.wYear, ST.wMonth, ST.wDay) + TimeSerial(ST.wHour, ST.wMinute, ST.wSecond)
    mFileTimeToDate = r
    
End Function

' return path with trailing backslash
Private Function mQualifyPath(ByRef sfolder As String) As String

    If Right$(sfolder, 1) <> "\" Then
        mQualifyPath = sfolder & "\"
    Else
        mQualifyPath = sfolder
    End If

End Function

' returns true if current file matches attributes and dot (..) properties
Private Function mMatchFilter() As Boolean

    Dim n As Long
    Dim r As Boolean
    
    ' check file attributes
    n = (mFileData.dwFileAttributes And mlngFilterAttributesMask)
    r = (n = (mlngFilterAttributes And mlngFilterAttributesMask))
    
    ' check for ".." and "."
    If mblnFilterDotFolders Then
        If (Me.FileName = "..") Or (Me.FileName = ".") Then
            r = False
        End If
    End If
    
    mMatchFilter = r

End Function

'*********'*********'*********'*********'*********'*********'*********'*********
' Class Initialize and Terminate Events
'*********'*********'*********'*********'*********'*********'*********'*********

' no initialization necessary

Private Sub Class_Terminate()
    If mhFind > 0 Then
        Call FindClose(mhFind)
    End If
End Sub
